﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace CSHowDoISamples
{
    class SelectFeaturesEditInteractiveOverlay : EditInteractiveOverlay
    {
        private PointStyle controlPointStyle;
        private PointStyle selectedControlPointStyle;
        private Collection<Feature> existingFeatures;
        private PointShape startPoint;
        Collection<int> ids;

        public SelectFeaturesEditInteractiveOverlay()
            : base()
        {
            this.CanAddVertex = false;
            this.CanDrag = true;
            this.CanRemoveVertex = false;
            this.CanResize = false;
            this.CanRotate = false;

            ids = new Collection<int>();

            existingFeatures = new Collection<Feature>();
        }

        public PointStyle ControlPointStyle
        {
            get { return controlPointStyle; }
            set { controlPointStyle = value; }
        }

        public PointStyle SelectedControlPointStyle
        {
            get { return selectedControlPointStyle; }
            set { selectedControlPointStyle = value; }
        }

        protected override void EndEditingCore(PointShape targetPointShape)
        {
            ClearAllControlPoints();
            CalculateCustomAllControlPoints();
            ShowAllControlPointLayers(true);
        }

        protected override InteractiveResult MouseDownCore(InteractionArguments interactionArguments)
        {
            InteractiveResult result = base.MouseDownCore(interactionArguments);
            startPoint = null;
            if (interactionArguments.MouseButton == MapMouseButton.Left)
            {
                if (EditShapesLayer.InternalFeatures.Count > 0)
                {
                    RectangleShape searchingArea = new RectangleShape(interactionArguments.WorldX - interactionArguments.SearchingTolerance, interactionArguments.WorldY + interactionArguments.SearchingTolerance, interactionArguments.WorldX + interactionArguments.SearchingTolerance, interactionArguments.WorldY - interactionArguments.SearchingTolerance);

                    foreach (Feature feature in DragControlPointsLayer.InternalFeatures)
                    {
                        BaseShape shape = feature.GetShape();
                        if (shape.IsWithin(searchingArea))
                        {
                            startPoint = new PointShape(interactionArguments.WorldX, interactionArguments.WorldY);
                        }
                    }
                    if (SetSelectedControlPoint(new PointShape(interactionArguments.WorldX, interactionArguments.WorldY), interactionArguments.SearchingTolerance))
                    {
                        startPoint = new PointShape(interactionArguments.WorldX, interactionArguments.WorldY);
                    }

                    if (startPoint == null)
                    {
                        result = base.MouseDownCore(interactionArguments);
                    }
                }
            }
            return result;
        }

        protected override InteractiveResult MouseMoveCore(InteractionArguments interactionArguments)
        {
            InteractiveResult result = new InteractiveResult();
            if (ControlPointType != ControlPointType.None)
            {
                if (ControlPointType == ThinkGeo.MapSuite.WpfDesktopEdition.ControlPointType.Drag)
                {
                    if (startPoint == null)
                    {
                        return result;
                    }
                    else
                    {
                        PointShape currentPoint = new PointShape(interactionArguments.WorldX, interactionArguments.WorldY);
                        double xOffset = currentPoint.X - startPoint.X;
                        double yOffset = currentPoint.Y - startPoint.Y;

                        Dictionary<int, Feature> features = new Dictionary<int, Feature>();

                        int index = 0;
                        if (ids.Count == 0)
                        {
                            foreach (Feature feature in EditShapesLayer.InternalFeatures)
                            {
                                foreach (Feature controlFeature in DragControlPointsLayer.InternalFeatures)
                                {
                                    PointShape oldPoint = controlFeature.GetShape() as PointShape;
                                    PointShape targetPoint = new PointShape(oldPoint.X + xOffset, oldPoint.Y + yOffset);
                                    features.Add(index, DragFeature(feature, oldPoint, targetPoint));

                                    index++;
                                }
                            }
                        }

                        foreach (var feature in features)
                        {
                            EditShapesLayer.InternalFeatures[feature.Key] = feature.Value;
                        }
                        CalculateCustomAllControlPoints();

                        startPoint = currentPoint.CloneDeep() as PointShape;
                        result.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
                        result.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;
                    }
                }
                else
                {
                    result = base.MouseMoveCore(interactionArguments);
                }
            }

            return result;
        }

        protected override InteractiveResult MouseClickCore(InteractionArguments interactionArguments)
        {
            InteractiveResult result = base.MouseClickCore(interactionArguments);
            if (result.DrawThisOverlay != InteractiveOverlayDrawType.Draw)
            {
                RectangleShape searchingArea = new RectangleShape(interactionArguments.WorldX - interactionArguments.SearchingTolerance, interactionArguments.WorldY + interactionArguments.SearchingTolerance, interactionArguments.WorldX + interactionArguments.SearchingTolerance, interactionArguments.WorldY - interactionArguments.SearchingTolerance);

                foreach (Feature feature in EditShapesLayer.InternalFeatures)
                {
                    BaseShape shape = feature.GetShape();
                    if (shape.Intersects(searchingArea) && DragControlPointsLayer.InternalFeatures.Count == 0)
                    {
                        CalculateCustomAllControlPoints();
                        result.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
                        result.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;
                    }
                }
            }

            return result;
        }

        public void CalculateCustomAllControlPoints()
        {
            ClearAllControlPoints();

            if (CanDrag) { CalculateCustomDragControlPoint(); }
            if (CanRotate) { CalculateRotateControlPoints(); }
            if (CanResize) { CalculateResizeControlPoints(); }
            if (CanReshape) { CalculateVertexControlPoints(); }

            ShowAllControlPointLayers(true);
        }

        private void ShowAllControlPointLayers(bool visiable)
        {
            DragControlPointsLayer.IsVisible = visiable;
            RotateControlPointsLayer.IsVisible = visiable;
            ResizeControlPointsLayer.IsVisible = visiable;
            ExistingControlPointsLayer.IsVisible = visiable;
        }

        protected void CalculateCustomDragControlPoint()
        {
            LineShape centerPointsLine = new LineShape();

            foreach (Feature feature in EditShapesLayer.InternalFeatures)
            {
                IEnumerable<Feature> dragControlPoints = CalculateDragControlPointsCore(feature);
                BaseShape baseShape = feature.GetShape();
                PointShape centerPointShape = baseShape.GetCenterPoint();

                if (double.IsNaN(centerPointShape.X) || double.IsNaN(centerPointShape.Y) || double.IsInfinity(centerPointShape.X) || double.IsInfinity(centerPointShape.Y))
                {
                    centerPointShape = baseShape.GetBoundingBox().UpperLeftPoint;
                }

                centerPointsLine.Vertices.Add(new Vertex(centerPointShape));
            }

            PointShape centerPointInCenterPoints = new PointShape(centerPointsLine.Vertices[0]);

            if (centerPointsLine.Vertices.Count == 2)
            {
                centerPointInCenterPoints = centerPointsLine.GetCenterPoint();
            }
            else if (centerPointsLine.Vertices.Count > 2)
            {
                centerPointInCenterPoints = centerPointsLine.GetBoundingBox().GetCenterPoint();
            }

            Feature addedFeature = new Feature(centerPointInCenterPoints);
            DragControlPointsLayer.InternalFeatures.Add(addedFeature);
        }
    }
}